local name, private = ...

Excavatinator = private.objects.base()

Excavatinator:RegisterEvent("ADDON_LOADED")

Excavatinator.isLoaded = false
Excavatinator.questList = {}
RequestArtifactCompletionHistory()

function Excavatinator:ADDON_LOADED(addon)
    if addon ~= name then return end
    private.artifacts:process()
    self:VerifyDB()
end

function Excavatinator:PLAYER_LOGOUT()
    self:SaveDB()
end

function Excavatinator:VerifyDB()
    -- Check that the saved data variable is a table; if not, overwrite it with a table
    if type(ExcavatinatorDB) ~= "table" then
        ExcavatinatorDB = {}
    end
    
    -- Do the same for linkedProjects
    if type(ExcavatinatorDB.linkedProjects) ~= "table" then
        ExcavatinatorDB.linkedProjects = {}
    end
    
    -- Do the same for framePosition
    if type(ExcavatinatorDB.framePosition) ~= "table" then
        ExcavatinatorDB.framePosition = { "CENTER", 0, 0, false }
    end
end

function Excavatinator:LoadDB()
    -- Load links
    for projectName, itemName in pairs(ExcavatinatorDB.linkedProjects) do
        private.links:addLink(itemName, projectName)
    end
end

function Excavatinator:SaveDB()
    if not self.isLoaded then return end
    
    -- Save links
    ExcavatinatorDB.linkedProjects = private.links.byProject -- The old addon kept links in this order. No good reason to change.
    
    -- Save frame position
    local point, _, _, x, y = self.raceFrame.frame:GetPoint()
    if not (point == "CENTER" and x == 0 and y == 0) then -- Somehow, the frame seems to reset to the center sometimes. No idea why.
        ExcavatinatorDB.framePosition = { point, x, y, self.raceFrame.isCollapsed }
    end
end

function Excavatinator:Load()
    -- Verify the saved data
    self:VerifyDB()
    
    -- Load saved data
    self:LoadDB()
    
    private.achievements:process()
    private.artifacts:process()
    self.raceFrame = private.objects.ui.raceFrame()
    self.raceFrame:init()
    self.raceFrame.frame:SetPoint(unpack(ExcavatinatorDB.framePosition))
    if ExcavatinatorDB.framePosition[4] then
        self.raceFrame:ShowContinent()
    end
    
    self:RegisterEvent("PLAYER_LOGOUT")
    
    self.isLoaded = true
end

function showExcavatinator() -- This is the slash command, so it can't have a self reference
    if InCombatLockdown() then
        print("You are in combat! Excavatinator cannot open.")
        return nil
    end
    if Excavatinator.isLoaded then
        Excavatinator.raceFrame:Show()
    else
        Excavatinator:Load()
        RequestArtifactCompletionHistory() -- Not sure if it took the first time; it did seem to have issues.
    end
end

SLASH_EXCAVATINATOR1 = '/arch'
SlashCmdList["EXCAVATINATOR"] = showExcavatinator